/*
	This file is part of lobj.

	lobj is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lobj is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lobj.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#ifndef LOBJ_OBJECT
#define LOBJ_OBJECT

#include "lobj.h"
#include "boxed.h"
#include <gobject/gvaluecollector.h>

/* Library functions, for intenal use */

gboolean priv_object_new(lua_State* L, GObject* object, gboolean need_unref);
gboolean priv_struct_new(lua_State* L, gpointer object, gboolean need_unref);
guint priv_timeout_add(gint priority, guint interval, Data* userdata);
guint priv_idle_add(gint priority, Data* userdata);

/* Value handling */

int lobj_value_push(lua_State* L);
int lobj_value_set(lua_State* L);

/* Object */

int lobj_object_get(lua_State* L);
int lobj_object_set(lua_State* L);
int lobj_object_connect(lua_State* L);
int lobj_object_block(lua_State* L);
int lobj_object_unblock(lua_State* L);
int lobj_object_disconnect(lua_State* L);
int lobj_object_unref(lua_State* L);
int lobj_object_ref(lua_State* L);
int lobj_object_notify(lua_State* L);
int lobj_object_thaw_notify(lua_State* L);
int lobj_object_freeze_notify(lua_State* L);

/* Struct */

int lobj_struct_new(lua_State *L);
int lobj_struct_to_string(lua_State* L);
int lobj_struct_gc(lua_State* L);

/* Used by other libraries, communicating via the Lua stack*/

int lobj_object_new(lua_State *L);
int lobj_object_eq(lua_State* L);
int lobj_object_gc(lua_State* L);
int lobj_closure_new(lua_State* L);
int lobj_object_to_string(lua_State* L);

#endif
