/*
	This file is part of lobj.

	lobj is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lobj is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lobj.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "random.h"

static int priv_rand_gc(lua_State* L)
{
	#ifdef IDEBUG
	g_debug("Garbage collecting a GRand!");
	#endif
	
	Object* obj = lua_touserdata(L, 1);
	g_rand_free((GRand*)obj->pointer);
	
	return 0;
}

int lobj_rand_new(lua_State* L)
{
	Object* obj = lua_newuserdata(L, sizeof(Object));
	obj->pointer = g_rand_new();
	obj->need_unref = TRUE; /* Not used, just for consistency */
	int top = lua_gettop(L);
	
	/* Shared metatable */
	if(luaL_newmetatable(L, "__lobjRaNdMt"))
	{
		lua_getfield(L, LUA_GLOBALSINDEX, "__lobjRegisterSpecial");
		lua_pushvalue(L, -2);
		lua_pushstring(L, "lobj");
		lua_pushstring(L, "Rand");
		lua_pushcfunction(L, priv_rand_gc);
		lua_call(L, 4, 0);
	}

	/* Set the object metatable */
	lua_setmetatable(L, top);
	
	return 1;
}

int lobj_rand_set_seed(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	Object* obj = lua_touserdata(L, 1);
	
	g_rand_set_seed((GRand*)obj->pointer, lua_tointeger(L, 2));
	
	return 0;
}

int lobj_rand_boolean(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gboolean res = g_rand_boolean((GRand*)obj->pointer);
	lua_pushboolean(L, res);
	
	return 1;
}

int lobj_rand_int(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	int top = lua_gettop(L);
	guint res = -1;
	
	if(top == 1)
	{
		res = g_rand_int((GRand*)obj->pointer);
	}
	else
	{
		luaL_checktype(L, 2, LUA_TNUMBER);
		luaL_checktype(L, 3, LUA_TNUMBER);
		res = g_rand_int_range((GRand*)obj->pointer, lua_tointeger(L, 2), lua_tointeger(L, 3));
	}
	
	lua_pushnumber(L, res);
	return 1;
}

int lobj_rand_double(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	int top = lua_gettop(L);
	gdouble res = -1;
	
	if(top == 1)
	{
		res = g_rand_double((GRand*)obj->pointer);
	}
	else
	{
		luaL_checktype(L, 2, LUA_TNUMBER);
		luaL_checktype(L, 3, LUA_TNUMBER);
		res = g_rand_double_range((GRand*)obj->pointer, lua_tonumber(L, 2), lua_tonumber(L, 3));
	}
	
	lua_pushnumber(L, res);
	return 1;
}
